// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

// mask texture
texture Mask<string uiname="Background";>;
sampler MaskSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Mask);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;

//mask transformation
float4x4 tMask: TEXTUREMATRIX <string uiname="Background Transform";>;


//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 MaskCd : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
    float4 maskCd : TEXCOORD1)
 {
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    //transform the mask texture cordinates
    Out.MaskCd = mul(TexCd, tMask);
    
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------



float4 ps_Difference(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskcol = tex2D(MaskSamp, In.MaskCd);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    float LumMask = maskcol.g;       //luminance in buffer in green channel only
    float TempCol = 0.0f;

    //Check Difference - Silhouette is black
    TempCol = abs(LumMask-LumCol);
    if (TempCol>0.5f) TempCol = 1.0f;

    //BW image out
    outColor.g= LumCol;
    outColor.r = TempCol;
    return outColor;
}

float4 ps_SeeDifference(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    float4 outColor = {0.0f,0.0f,0.0f,0.0f};

    //BW image out
//    outColor.rgb= col.r;
    if (col.r > 0.05f) outColor.rgba = 1.0f;
    return outColor;
}

float4 ps_SeeOverlap(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    float4 outColor = {0.0f,0.0f,0.0f,0.0f};
    float tempCol = 0.0f;

    //BW image out
    if (col.g>0.5f && col.r<0.5f) tempCol = 0.0f;
    outColor.rgb= tempCol;
    if (col.r > 0.05f) outColor.a = 1.0f;
    return outColor;
}

float4 ps_SeeDifferenceInside(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    float4 outColor = {0.0f,0.0f,0.0f,0.0f};
    float tempCol = 0.0f;

    //BW image out
    //get out yellow -> movement toward new position
    if (col.g>0.5f && col.r>0.5f) {
       tempCol = 1.0f;
       outColor.a = 1.0f;
    }
       outColor.rgb= tempCol;
    return outColor;
}
// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------


technique Difference
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_Difference();
    }
}

technique SeeDifference
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_SeeDifference();
    }
}

technique SeeOverlap
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_SeeOverlap();
    }
}

technique SeeDifferenceInside
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_SeeDifferenceInside();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (SrcSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
